#ifndef AK_ENG_INF_H
#define AK_ENG_INF_H

#include "cap.h"

/**
  * Provides a C compatible interface to call the akenti policy engine
  * 
  * @see AkentiEngine
  * @see CapabilityCertificate
  *
  * @version 1.1 03/12/02
  * @author Abdelilah Essiari
  */

#ifdef __cplusplus
extern "C" {
#endif

/**
  * Initializes the Akenti policy engine. This call should be called
  * once during the lifetime of a program.
  *
  * @param confFile  name of the Akenti configuration file.
  * @param status a returned string containing any error messages
  *
  * @return 0 if the initialization was successful.
  */
int akentiInit(const char *confFile, char**status);

/**
  *  Given an authenticated principal name which consists of an x509
  *  distinguished name for a user, and the x509 distinguished name of
  *  the Certificate Authority which issued the user's x509 cert, and
  *  the name of the resource, returns a capability certificate if any
  *  rights are granted. 
  *
  *  Note: akentiInit must be called prior to this call.
  *
  *  @param subjectDN  [in]  the user for whom we are checking access
  *  @param subjectCA  [in]  the CA that issued the subject's cert 
  *  @param resource   [in]  name of the resource for which access is checked
  *  @param cert       [out] a pointer to a capability certificate
  *  @param AKMessage [out] a message grouped with a status code
  *
  *  @return 0 if succesful (user is not denied access)
  *
  *  @see freeAKMessage
  */
int checkAccess(const char *subjectDN,  
                const char *subjectCA, 
	        const char *resource, 
                CapCert* cert,
                AKMessage* mesg);

/**
  *  Given a file containing a PEM encoded x509 certificate for a user, and
  *  the name of the resource, returns a capability certificate if any
  *  rights are granted. 
  *
  *  Note: akentiInit must be called prior to this call.
  *
  *  @param subjectCertFile  [in] user's X.509 certificate  
  *  @param resource         [in] name of the resource for which access 
  *                             is to be checked
  *  @param cert             [out] a pointer to a capability certificate
  *  @param AKMessage        [out] a message grouped with a status code
  *
  *  @return 0 if successful (user is not denied access)
  *
  *  @see freeAKMessage
  */
int checkAccessByCertFile(const char *subjectCertFile, 
                          const char *resource, 
                          CapCert* cert,
                          AKMessage* mesg);

/**
  *  Given a string containing a PEM encoded x509 certificate for a user, and
  *  the name of the resource, returns a capability certificate if any. 
  *
  *  Note: akentiInit must be called prior to this call.
  *
  *  @param subjectCert      [in] PEM encoded string containing user's X.509 
  *                             certificate chain
  *  @param resource         [in] name of the resource for which access 
  *                             is to be checked
  *  @param cert             [out] a pointer to a capability certificate
  *  @param AKMessage        [out] a message grouped with a status code
  *
  *  @return 0 if succesful (user is not denied access)
  *
  *  @see freeAKMessage
  */
int checkAccessByCertString(const char *subjectCert, 
                          const char *resource, 
                          CapCert* cert,
                          AKMessage* mesg);

#ifdef __cplusplus
}
#endif
#endif
