#ifndef ATTRIBUTE_CERTIFICATE_IMPL_HPP
#define ATTRIBUTE_CERTIFICATE_IMPL_HPP

#include <vector>
#include "AkentiCertificateImpl.hpp"
#include "Condition.hpp"

using std::vector;

namespace Akenti {

/**
  * This class provides the implementation of an <i>attribute</i>
  * certificate.
  *
  * This certificate, once verified, will attest that the user
  * satisfies the attribute/value pair contained in this certificate.
  * This certificate may contain a </i>Condition</i> that will 
  * need to be evaluated before the user is said to satisfy 
  * the attribute/value pair.
  * 
  * These certificates are usually collected by the engine when
  * evaluating the boolean expression contained inside a
  * <i>UseCondition</i> certificate.
  * 
  * <p>
  * The format for an attribute certificate is:
  * <pre>
  * AttributeCertificateImpl { 
  *   type           // <i>Attribute</i>
  *   version        //  certificate's version  number
  *   id             //  certificate's unique id 
  *   issuer         //  issuer and signer of certificate 
  *   notBefore      //  start validity time
  *   notAfter       //  end validity time
  *   signatureAlg   //  signature algorithm
  *   attribute      //  attribute's name  
  *   value          //  attribute's value 
  *   condition      //  constraint and the corresponding attributeInfos
  *   signature      //  digital signature 
  * } 
  * </pre>
  *
  * <p>
  * Generating an attribute  certificate:
  *
  * <blockquote>
  *   AttributeCertGenerator gen;
  *
  *   gen.setIssuer(..);
  *   gen.setAttribute(..);
  *   ... 
  *   gen.sign(...).
  *   
  *   AkentiCertificate cert = gen.generate();
  * </blockquote>
  *
  * @see AttributeCertGenerator 
  * @see Condition 
  * @see AkentiPrincipal
  *
  * @version 1.1 00/05/25
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  */ 

class AttributeCertificateImpl : public AkentiCertificateImpl {

public:

  friend class AttributeCertGenerator;

  /**
    * Default Constructor.
    */
  AttributeCertificateImpl();

  /**
    * Default destructor
    */
  ~AttributeCertificateImpl();

  /**
    */
  string getHashString() const;

  /**
    */
  const AkentiPrincipal& getPrincipal() const;

  /**
    */
  const string& getAttribute() const;

  /**
    */
  const string& getValue() const;

  /**
    */
  bool containsCondition() const;

  /**
    */
  const Condition& getCondition() const;

  /**
    */
  const string& getConstraint() const;

  /**
    */
  const vector<AttributeInfo>& getAttributeInfos() const;

protected:

  /**
    */
  AttributeCertificateImpl* newInstance() const;

  /**
    */
  void doWrite(AkentiOutputStream& drain) const;

  /**
    */
  void doRead(AkentiInputStream& source);

  /**
    */
  void doToXML(DOMElement* attrCertInfo) const;

  /**
    */
  void doFromXML(const DOMElement* attrCertInfo);

  /**
    */
  void doParamString(ostream& os) const;

  /**
    */
  void doToText(int indentAmt, ostream& os) const;

private:

  /**
    */
  void setPrincipal(const AkentiPrincipal& principal);

  /**
    */
  void setAttribute(const string& attribute);

  /**
    */
  void setValue(const string& value);

  /**
    */
  void setCondition(const Condition& condition);

  AkentiPrincipal principal;

  string attribute;

  string value;

  Condition* condition;

};

} // end of name space

#endif
