#ifndef AK_BASE_64_HPP
#define AK_BASE_64_HPP

#include "ref_ptr.hpp"
#include "Base64Impl.hpp"
#include "Provider.hpp"

/**
  * A <i>Base64</i> object is a <i>reference-count</i>
  * pointer that wraps a <i>Base64Impl</i> object.
  *
  * <pre>
  *<b>Usage:</b>
  *     try {
  *       string mesg = "hello there";
  *       string enc = Base64::encode(mesg);
  *       string data = Base64::decode(enc);
  *     } catch(Exception &ex) {.....}
  *
  * or <b>Another Usage:</b>
  *	try {
  *	  Base64 encoder = Base64::getInstance();
  *      
  *       encoder->encodeInit();
  *       encoder->encodeUpdate("hello");
  *       encoder->encodeUpdate(" ");
  *       encoder->encodeUpdate("there");
  *
  *       string enc = encoder->encode();
  * 
  *       Base64 decoder = Base64::getInstance();
  *      
  *       decoder->decodeInit();
  *       decoder->decodeUpdate(enc);
  *       decoder->decodeInit();
  *       decoder->encodeUpdate(enc);
  *
  *       string data = decoder->decode();
  *     } catch(Exception &ex) {.....}
  * </pre>
  *
  * @see ref_ptr
  * @see Base64Impl
  * @see Provider
  * @see ProviderImpl#getBase64
  *
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  * @author Xiang Sun
  */

namespace Akenti {

class Base64 : public ref_ptr<Base64Impl> {

public:

  /**
    * Constructs a <i>Base64</i> given a <i>Base64Impl</i> object.
    * Typically the <i>getInstance()</i> method is
    * used to create an instance of this class rather than the constructor.
    *
    * @param impl wrapped <i>Base64Impl</i> object
    * @see Base64Impl
    * @see getInstance() 
    * @see ref_ptr  
    */
  Base64(Base64Impl* impl = NULL) : ref_ptr<Base64Impl> (impl) {
  }

  /**
    * Decerements the ref_cnt, and destroys the object when the ref_cnt is 0.
    *
    * @see ref_ptr
    */
  ~Base64(){
  }

  /**
    * Returns a <i>Base64</i> object that provides the implementation
    * of the base64 encoding algorithms that is installed in the Provider.
    *
    * @return A Base64 object
    * @exception Error if provider is not installed
    * @see Provider#getProvider()
    * @see ProviderImpl#getBase64()
    */ 
  static Base64 getInstance() {
    Provider& po = Provider::getProvider();

    return po->getBase64();
  }

  /**
    * A static method that returns a encoded value given
    * the string to be encoded specified by <i>data</i>.
    *
    * @param data string to be encoded
    * @exception Error if provider has not been installed
    * @see getInstance()
    * @see Base64Impl#encodeInit()
    * @see Base64Impl#encodeUpdate(string)
    * @see Base64Impl#encode()
    */
  static string encode(const string& data) {
    Base64 encoder = getInstance();
    
    encoder->encodeInit();
    encoder->encodeUpdate(data);
    return encoder->encode();
  }
  
  /**
    * A static method that returns a decoded value given
    * the string to be decoded specified by <i>enc</i>.
    *
    * @param enc string to be decoded  
    * @exception Error if provider has not been installed
    * @exception SecurityException if decoding fails 
    * @see getInstance()
    * @see Base64Impl#decodeInit()
    * @see Base64Impl#decodeUpdate(string)
    * @see Base64Impl#decode()
    */
  static string decode(const string& enc) {
    Base64 decoder = getInstance();
    
    decoder->decodeInit();
    decoder->decodeUpdate(enc);
    return decoder->decode();
  }

};

} // end of namespace Akenti

#endif
