#ifndef CAPABILITY_CERTIFICATE_IMPL_HPP
#define CAPABILITY_CERTIFICATE_IMPL_HPP

#include <vector>
#include "AkentiCertificateImpl.hpp"
#include "ConditionalActions.hpp"
#include "PublicKey.hpp"

using std::vector;

namespace Akenti {

/**
  * This class provides the implementation of a <i>capability</i>
  * certificate.
  *
  * <p>
  * The format for an attribute certificate is:
  * <pre>
  * AttributeCertificateImpl {
  *   type                  // <i>Capability</i>
  *   version               //  certificate's version  number
  *   id                    //  certificate's unique id
  *   issuer                //  issuer and signer of certificate
  *   notBefore             //  start validity time
  *   notAfter              //  end validity time
  *   signatureAlg          //  signature algorithm
  *   principal             //  principal
  *   publicKey             //  principal's public key (version 2)
  *   resource              //  name of resource
  *   unConditionalActions  //  actions with no conditions attached to them
  *   conditionalActions    //  actions that would be granted once the 
  *                         //    condition attached to them evalutes to true
  *   signature             //  digital signature
  * }
  * </pre>
  *
  * <p>
  * Generating a capability certificate:
  *
  * <blockquote>
  *   CapabilityCertificateGenerator gen;
  *
  *   gen.setIssuer(..);
  *   ...
  *   gen.sign(...).
  *
  *   AkentiCertificate cert = gen.generate();
  * </blockquote>
  *
  * @see CapabilityCertGenerator
  * @see ConditionalActions
  * @see AkentiPrincipal
  *
  * @version 1.1 00/05/25
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  */

class CapabilityCertificateImpl : public AkentiCertificateImpl {

public:

  /**
    * Default constructor.
    */
  CapabilityCertificateImpl();

  /**
    * Default destructor.
    */
  ~CapabilityCertificateImpl();

  /**
    */
  string getHashString() const;

  /**
    */
  const string& getResource() const ;

  /**
    */
  const AkentiPrincipal& getPrincipal() const;

  /**
    */
  const PublicKey& getPublicKey() const;

  /**
    */
  const vector<string>& getUnConditionalActions() const;

  /**
    */
  const vector<ConditionalActions>& getConditionalActions() const;

protected:

  /**
    */
  void doWrite(AkentiOutputStream& drain) const;

  /**
    */
  void doRead(AkentiInputStream& source);

  /**
   */
  void doToXML(DOMElement* capabilityCertInfo) const;

  /**
    */
  void doFromXML(const DOMElement* capabilityCertInfo);

  /**
    */
  CapabilityCertificateImpl* newInstance() const;

  /**
    */
  void doParamString(ostream& os) const;

  /**
    */
  void doToText(int indentAmt, ostream& os) const; 

private:

  friend class CapabilityCertGenerator;

  /**
    */
  void setResource(const string& resource);

  /**
    */
  void setPrincipal(const AkentiPrincipal& principal);

  /**
    */
  void setPublicKey(const PublicKey& publicKey);

  /**
    */
  void 
  setUnConditionalActions(const vector<string>& unConditionalActions);

  /**
    */
  void 
  setConditionalActions(const vector<ConditionalActions>& conditionalActions);

  string resource;

  AkentiPrincipal principal;
  
  PublicKey publicKey;

  vector<string> unConditionalActions;

  vector<ConditionalActions> conditionalActions;

};

} // end of name space

#endif
