#ifndef CERTIFICATE_VERIFIER_HPP
#define CERTIFICATE_VERIFIER_HPP

#include <string>
#include <vector>
#include "LogInterface.hpp"
#include "AKCertHolder.hpp"
#include "AkentiX509Certificate.hpp"
 
using std::string;
using std::vector;
using namespace Akenti;

namespace Akenti {

class URL;

/** 
  * A <i>CertificateVerifier</i> is used to verify <i>AkentiCertificate</i>
  * certificates. 
  *
  * @version 1.1 00/05/01
  * @author Abdlelilah Essiari 
  * @author Srilekha Mudumbai 
  */

class CertificateVerifier : public LogInterface {

public:

  /**
    * Verifies the <i>AkentiCertificate</i> specified by <i>cert</i>.
    * The verification succeds if the X509 identity certificate of the
    * <i>cert</i>'s issuer is found to be valid and trusted. 
    *
    * Algorithm to fetch the issuer's X509 identity certificate:
    *
    * <p>
    *
    * 1) trusted ca certs:<br> 
    *    We first look to see if the issuer's cert is one these trusted
    *    certficates. If so we are done. 
    * <p>
    *
    * 2) cache:<br> 
    *    We then query the cache for an X509 certificate for that issuer. 
    *    Recall the cache only returns certificates that have not been 
    *    cached longer than <i>cacheInterval</i>. Furthermore, the cache
    *    never returns a certificate that has expired.
    *    So if the issuer's certificate is found in the cache,
    *    we only need to consider two cases:
    * <p>      
    *      a) Issuer's cert has been verified. We are done.<br>
    *      b) Issuer's cert has not yet been verified. We verify <br>
    *         it by starting at step one. 
    * <p>
    *
    * 3) directories:<br>
    *    We search the identity directories for the issuer's cert. We cache
    *    each certificate when it is found. We then try to use the public
    *    key in the cert to verify our target certificate. The search stops
    *    when the verification is successful. 
    *    We now proceed to verify the issuer's cert by starting at step one.  
    *
    * @param cert AkentiCertificate cert to be verified. 
    * @param dirs vector<Directory> directories to loolk up X509 certs. 
    * @param cacheInterval int the allowable duration of a certificate 
    *        in the cache. 
    *
    * @return true if certificate verifies, false otherwise.
    *
    * @see verify(AkentiCertificate, AkentiCertificate&)
    * @see CertificateCache
    * @see AkentiCertificate
    */
  bool 
  verify(const AkentiCertificate& cert);

  /**
    *
    */
  bool 
  verify(AKCertHolder& certHolder);

  /**
    */
  bool
  verify(const AkentiPrincipal& ap, AkentiX509Certificate& cert);

private:

  /* 
   * Only the runtime class will create and destroy this object.
   */
  friend class AKRuntime;

  /**
    * Constructs this object. 
    */
  CertificateVerifier(AKRuntime* runtime);

  /**
    * Destroys this object.
    */
  ~CertificateVerifier();
  
  /**
    * Disallows copying.
    */
  CertificateVerifier(const CertificateVerifier&);

  /**
    * Disallows assignement.
    */
  void operator=(const CertificateVerifier&);

  /**
    * Determines whether <i>issuerCert</i> was used to "sign" 
    * <i>issuedCert</i>. 
    *
    *   1) compare the name of the issuer of issuedCert 
    *      to the subject of the <i>issuedCert</i>. 
    *      Under normal circumstances I would not have to say anything
    *      more about this name comparison, but ...
    *      We use a rather weak form of equality. 
    *        Let dn1 be the name of the subject of issuerCert
    *        Let dn2 be name of the issuer of issuedCert
    *        We are satisfied if every component in dn1 is in dn2.
    *   2) We check to see if the issuer's public key corresponds to the
    *      private key that was used to sign <i>issuedCert</i>. This is done
    *      calling the AkentiCertificate verify method with the public key
    *      which in turn calls the verify method of the Signature algorthim
    *      with the signable part of the certificate, the signature and
    *      the public key.
    *
    * @return true if dn1 is a subset of dn2 and if the public key passes 
    *         the second test, false otherwise.
    *
    * @see AkentiCertificate
    */
  bool 
  verify(const AkentiCertificate& issuerCert, 
         const AkentiCertificate& issuedCert);
  
  /**
    * Helper function.
    */
  AKCertHolder 
  getIssuerCert(const AkentiCertificate& cert, int& caLevel);
  
  /*
   *
   */
  AKRuntime *akRuntime;

};
 
} // end of name space

#endif
