#ifndef POLICY_CERTIFICATE_IMPL_HPP
#define POLICY_CERTIFICATE_IMPL_HPP

#include <vector>
#include "AkentiCertificateImpl.hpp"
#include "UseCondIssuerGroup.hpp"
#include "CertificateAuthorityInfo.hpp"

using std::vector;

namespace Akenti {

/**
  * A Policy certificate specifies the use-condition
  * certificates issuers, directories where to search for
  * attribute and identity certificates.
  *
  * <p>
  * The encoding for a policy certificate is:
  *
  * <pre>
  * <b>PolicyCertificateImpl</b> { 
  *    type                   // <i>Policy</i>
  *    version                // certificate's version number
  *    id                     // certificate's unique id
  *    issuer                 // issuer and signer of certificate
  *    notBefore              // start validity date
  *    notAfter               // end validity date
  *    signatureAlg           // name of algorithm used to sign this certificate
  *    resource               // name of the resource to which this policy applies
  *    certAuthInfos          // (0 or more) trusted cas and their infos
  *    ucIssuerGroups         // (1 or more) list of groups of stakeholders and their directories
  *    attributeDirs          // (0 or more) directories to search for attribute certificates
  *    cacheTime              //  
  *    signature              // digital signature of the certificate
  * }
  * </pre>
  *
  * @see CertificateAuthorityInfo 
  * @see UseCondIssuerGroup
  * @see Directory
  *
  * @version 1.1 01/08/15
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari 
  */

class PolicyCertificateImpl : public AkentiCertificateImpl {

public:

  /**
    * Default Constructor.
    */
  PolicyCertificateImpl();

  /**
    * Destroys this object.
    */
  ~PolicyCertificateImpl();

  /**
    * Returns the name of the resource.
    * 
    * @see AkentiCertificateImpl#hashCode()
    */
  string getHashString() const;

  /**
    * Returns the name of the resource.
    *
    **/
  const string& getResource() const;

  /**
    */
  bool
  containsCertificateAuthorityInfos() const;

  /**
    * Returns <i>CertificateAuthority</i> infos.
    * 
    * @see CertificateAuthorityInfo
    */
  const vector<CertificateAuthorityInfo>&
  getCertificateAuthorityInfos() const;

  /**
    * Returns <i>UseConditionIssuer</i> groups.
    *
    * @see UseCondIssuerGroup
    */
  const vector<UseCondIssuerGroup>& 
  getUseCondIssuerGroups() const;

  /**
    * Returns the directories used to find 
    * <i>attribute</i> certificates.
    *
    * @see Directory
    */
  const vector<Directory>& 
  getAttributeDirectories() const;

  /**
    * Returns the caching time in seconds.
    *
    **/
  long getCacheTime() const;

protected:

  /**
    */
  void doWrite(AkentiOutputStream& drain) const;

  /**
    */
  void doRead(AkentiInputStream& source);

  /**
   */
  void doToXML(DOMElement* attrCertInfo) const;

  /**
    */
  void doFromXML(const DOMElement* attrCertInfo);

  /**
    */
  PolicyCertificateImpl* newInstance() const;

  /**
    */
  void doParamString(ostream& os) const;
  
  /**
    */
  void doToText(int indentAmt, ostream& os) const; 

private:

  friend class PolicyCertGenerator;

  /**
    **/
  void 
  setResource(const string& resource);

  /**
    */
  void 
  setCertificateAuthorityInfos(const vector<CertificateAuthorityInfo>& caInfos);

  /**
    */
  void
  setAttributeDirectories(const vector<Directory>& dirs);

  /**
    */
  void 
  setUseCondIssuerGroups(const vector<UseCondIssuerGroup>& ucGroups);

  /**
    */
  void setCacheTime(long cacheTime);

  string resource;

  vector<CertificateAuthorityInfo> certAuthInfos;

  vector<Directory> attributeDirs;

  vector<UseCondIssuerGroup> ucIssuerGroups;

  long cacheTime;

}; 
 
} // end of name space

#endif
