#ifndef AK_SSL_CONSTANTS_HPP 
#define AK_SSL_CONSTANTS_HPP

namespace Akenti {

/**
  * A structure to define the constants used by the security provider. 
  * Currently only values used by SSLeay and OpenSSL are defined, 
  * since those are the only providers we have.
  * 
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  */

struct SSLConstants {
   
  /**
    * Version constants.
    *
    * @type Version
    * 
    * Values of <i>Version</i> are: <br>
    * <blockquote>
    *   "SSL2", <br> 
    *   "SSL3", <br>
    *   "TLS1"  <br>
    * </blockquote>
    * 
    */
  enum Version { SSL2, SSL3, TLS1 }; 
   
  /**
    * These constants describe the encoding of identity certificates
    * or private keys contained in the certificates.
    *
    * @type FileType
    *
    * Values of <i>FileType</i> are: <br> 
    * <blockqoute>
    *   "PEM", <br> 
    *   "ASN1" <br>
    * </blockqoute>
    */
  enum FileType { PEM, ASN1 };

  /** 
    * These constants are used as arguments to 
    * <i>SecureSocketImpl#setVerify(int)</i>. <br>
    *
    * <blockqoute>
    * Possible values: <br>                                VERIFY_NONE  (0) <br>
    *                                                      VERIFY_PEER  (1) <br>
    *                        VERIFY_PEER | VERIFY_FAIL_IF_NO_PEER_CERT  (3) <br>
    *                                 VERIFY_PEER | VERIFY_CLIENT_ONCE  (5) <br>
    *   VERIFY_PEER | VERIFY_FAIL_IF_NO_PEER_CERT | VERIFY_CLIENT_ONCE  (7) <br>
    * </blockqoute>
    * 
    */

  /**
    * Do not verify peer.
    */
  static const int VERIFY_NONE                  = 0x00;

  /**
    * Verify peer if possible. 
    */
  static const int VERIFY_PEER                  = 0x01;

  /** 
    * Verify peer, fail otherwise.
    */
  static const int VERIFY_FAIL_IF_NO_PEER_CERT  = 0x02;

  /** 
    * Used by server for better performance.
    */
  static const int VERIFY_CLIENT_ONCE           = 0x04;
  
  /**
    * Ciphers we are using.  Definitions moved to SSLConstants.ccp to
    * keep Sun CC compiler happy.  (Why these const vars are needed in
    * the first place, I'm not sure.  Seems like they are only used in
    * the CIPHERS[] anyway...)
    */
  static const char* const RSA_EXPORT1024_WITH_RC4_56_MD5;

  /**
    */
  static const char* const RSA_EXPORT1024_WITH_RC2_CBC_56_MD5;

  /**
    */
  static const char* const RSA_EXPORT1024_WITH_DES_CBC_SHA;

  /**
    */
  static const char* const DHE_DSS_EXPORT1024_WITH_DES_CBC_SHA;

  /**
    */
  static const char* const RSA_EXPORT1024_WITH_RC4_56_SHA;

  /**
    */
  static const char* const DHE_DSS_EXPORT1024_WITH_RC4_56_SHA;

  /**
    */
  static const char* const DHE_DSS_WITH_RC4_128_SHA;

  /**
    * Values of <i>CIPHERS</i> are initialized in SSLConstants.cpp and are:
    * 
    * <blockquote>
    *   RSA_EXPORT1024_WITH_RC4_56_MD5,     <br>
    *   RSA_EXPORT1024_WITH_RC2_CBC_56_MD5, <br>
    *   RSA_EXPORT1024_WITH_DES_CBC_SHA,    <br>
    *   DHE_DSS_EXPORT1024_WITH_DES_CBC_SHA,<br>  
    *   RSA_EXPORT1024_WITH_RC4_56_SHA,     <br>
    *   DHE_DSS_EXPORT1024_WITH_RC4_56_SHA, <br>
    *   DHE_DSS_WITH_RC4_128_SHA            <br>
    * </blockquote> 
    */
  static const char* const CIPHERS[];

  /**
    */
  static const int REVOKED = 1;

  /**
    */
  static const int NOT_REVOKED = 2;

  /**
    */
  static const int UNKNOWN = 3;

protected:

  SSLConstants() {
  }

};

} // end of namespace Akenti

#endif

