#ifndef AK_SSL_SERVER_SOCKET_HPP
#define AK_SSL_SERVER_SOCKET_HPP

#include "ServerSocket.hpp"
#include "SSLContext.hpp"
#include "SSLSocket.hpp"
#include "Provider.hpp"

namespace Akenti {

/** 
  * <i>SSLServerSocket</i> class - derived from <i>ServerSocket</i> but has a 
  * <i>SSLcontext</i> associated with it. Inherits most of its methods from 
  * ServerSocket - i.e. setting the host, setting/getting the port, 
  * connection and binding, listen and accept.
  * 
  * @see ServerSocket
  * @see SSLContext
  * @see SSLSocket
  * @see Provider
  *
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  */  

class SSLServerSocket : public ServerSocket {
public:

  /**
    * Constructs a SSLServerSocket from a SecureSocketImpl.
    *
    * @param SecureSocketImpl probably created by the newInstance method
    *        of the concrete SecureSocketImpl.
    */  
  SSLServerSocket(SecureSocketImpl* impl = NULL) : ServerSocket(impl) { 
  }

  /**
    * Constructs a SSLServerSocket from the Provider default SSLServerContext
    * and  default SecureSocketImpl.
    *
    * @param  port to on which the socket will listen
    * @see Provider
    */
  SSLServerSocket(int port) {
    Provider& po = Provider::getProvider();
    const SSLContext& context = SSLContext::getDefaultSSLServerContext();

    impl = po->createSecureSocketImpl(context);
    impl->setPort(port);
  }

  /**
    * Constructs a SSLServerSocket from a Server SSLContext and the Provider
    * default SecureSocketImpl.
    *
    * @param ctx a server SSLContext
    * @param  port port on which the socket will listen
    * @exception Error if the input SSLcontext is not a server context.
    * @see Provider
    */
  SSLServerSocket(const SSLContext& ctx, int port) { 
    if (ctx->isClientContext()) {
      throw Error("ServerSocket:constructor:invalid context");
    }

    Provider& po = Provider::getProvider();

    impl = po->createSecureSocketImpl(ctx);
    impl->setPort(port);
  }

  /**
    * Destroys this object.
    */
  ~SSLServerSocket() {
  }

  /**
    * Does  the accept - needs more explanation.
    *
    * @return  an new SSLSocket on which an accept has been done.
    */
  SSLSocket accept() {
    SecureSocketImpl* dimpl = (SecureSocketImpl*) impl.get();

    SecureSocketImpl* newImpl = dimpl->newInstance(
      dimpl->getSSLContext()
    );

    SSLSocket ret(newImpl);

    dimpl->doAccept(newImpl);
    return ret;
  }

  /**
    * Return the security  context associated with this socket.
    *
    * @return - the SSLContext associated with this socket.
    */
  const SSLContext& getSSLContext() {
    SecureSocketImpl* dimpl = (SecureSocketImpl*) impl.get();

    return dimpl->getSSLContext();
  }

private:

  friend class ServerSocket;

  typedef SecureSocketImpl* ImplType;

};

} // end of namespace  

#endif

