#ifndef USE_COND_CERTIIFICATE_IMPL_HPP
#define USE_COND_CERTIIFICATE_IMPL_HPP

#include <vector>
#include "AkentiCertificateImpl.hpp"
#include "Condition.hpp"

using std::vector;

namespace Akenti {

class AttributeInfo;

/**
  * These certificates are mainly used by the stakeholders 
  * or resource owners to place a use condition on the 
  * access/usage of a resource. They also specify actions
  * that will be awarded if the condition is satisfied.
  *
  * <p>
  * The encoding of use-condition certificate is: 
  *
  * <pre><b>UseCondCertificateImpl</b> { 
  *     type             // <i>UseCondition</i>
  *     version          // certificate's version number
  *     id               // certificate's unique id
  *     issuer           // issuer and signer of certificate
  *     notBefore        // start validity date
  *     notAfter         // end validity date
  *     signatureAlg     // name of algorithm used to sign this certificate
  *     resource         // name of resource UC applies to 
  *     localScope
  *     critical
  *     condition        // a use-condition that user must satisfy 
  *     actions          // actions awarded to the user if condition is 
  *                      //   satisfied 
  *     signature        // digital signature of the certificate
  * } </pre>
  *
  * <p>
  * Generating a UC certificate <BR>
  *
  * <blockquote>
  *      UseCondCertGenerator gen;
  *
  *      gen.setIssuer(..);      
  *      gen.setAttributeInfos(..);     
  *      ...                         
  *      gen.sign(...)              
  *
  *      AkentiCertificate cert = gen.generate()
  * </blockquote>
  *
  * @see AkentiCertificateImpl
  * @see AkentiCertificate
  * @see AkentiPrincipal
  * @see UseCondCertGenerator
  *
  * @version 1.1 00/05/25
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  */


class UseCondCertificateImpl : public AkentiCertificateImpl {

public:

  /**
    * Default constructor 
    */
  UseCondCertificateImpl();

  /**
    * Default destructor
    */
  ~UseCondCertificateImpl();

  /**
    * Returns a string whose hash will be used to locate this certificate.
    */
  string getHashString() const;

  /**
    * Returns the name of the resource.
    *
    * @return string the resource of this certificate
    */
  const string& getResource() const;

  /**
    * Returns <i>true</i> if this certificate does not
    * apply to sub resources.
    */
  bool isScopeLocal() const;

  /**
    * Returns <i>true</i> if this certificate must be satisfied.
    */
  bool isCritical() const;

  /**
    * Returns the <i>Condition</i>.
    *
    * @see Condition 
    */
  const Condition& getCondition() const;

  /**
    *
    *
    * @see Condition#getConstraint() 
    */
  const string& getConstraint() const;
  
  /**
    * Returns the list of <i>AttributeInfo</i>s.
    *
    * @see Condition#getAttributeInfos() 
    */
  const vector<AttributeInfo>& getAttributeInfos() const;

  /**
    * Returns the list of actions.
    *
    */
  const vector<string>& getActions() const;

protected:

  /**
    **/
  UseCondCertificateImpl* newInstance() const;

  /**
    * Writes this certificate object into an AkentiOutputStream
    *
    * @param drain the AkentiOutputStream this certificate writes to
    * @exception IOException if writing fails
    * @see AkentiOutputStream
    * @see IOException
    */
  void doWrite(AkentiOutputStream& drain) const; 

  /**
    * Reads this certificate object from an AkentiInputStream
    *
    * @param source the AkentiInputStream being read to this certificate
    * @exception ParsingException if parsing the distinguished name fails
    * @exception IOException if reading fails
    * @exception Error if have no issuers
    * @see AkentiInputStream
    * @see DistinguishedName
    * @see ParsingException
    * @see IOException
    * @see Error
    */
  void doRead(AkentiInputStream& source); 

  /**
    */
  void doToXML(DOMElement* ucCertInfo) const;

  /**
    */
  void doFromXML(const DOMElement* ucCertInfo);

  /**
    * For debugging.
    */
  void doParamString(ostream& os) const;

  /**
    */
  void doToText(int indentAmt, ostream& os) const;


private:

  friend class UseCondCertGenerator;

  /**
    * Sets resouce for this certificate.
    *
    * @param resource being set to this certificate
    */
  void setResource(const string& resource);

  /**
    */
  void setLocalScope(bool isScopeLocal);

  /**
    */
  void setCritical(bool isCritical);

  /**
    * Sets the constarint for this certificate.
    *
    * @param constraint being set to this certificate
    */
  void setCondition(const Condition& condition);

  /**
    * Adds action for this certificate.
    *
    * @param action being added to this certificate
    */ 
  void setActions(const vector<string>& actions);

  string resource;

  bool localScope;

  bool critical;

  Condition condition;

  vector<string> actions;

  vector<DistinguishedName> subjectCAS;

};

} // end of name space

#endif
