#include <iostream>
#include <fstream> 
#include <string> 
#include <vector> 
#include <set> 
#include <unistd.h>
#include <map> 
#include "AkentiEngine.hpp"
#include "CertificateFactory.hpp"
#include "CertificateChain.hpp"
#include "AkentiInit.hpp"
#include "CapabilityCertificate.hpp"
#include "AkentiMessage.hpp"
#include "PolicyContext.hpp"
#include "Policy.hpp"
#include "PolicyCertificate.hpp"
#include "AttributeCertificate.hpp"
#include "UseCondCertificate.hpp"
#include "ConditionalActions.hpp"
#include "File.hpp"
#include "Exception.hpp"
#include "Error.hpp"

using namespace Akenti;
using namespace std;

/**
  * <pre>
  * <b>CheckAccess:</b>
  *         Use this program to check a user's access to a resource.
  * </pre>
  */
namespace CheckAccess {

void
printActions(const vector<string>& actions) {
  cout << "Allowed actions:";
 
  int size = actions.size();
 
  if (size == 0) {
    cout << "NONE" << endl;
    return;
  }
 
  cout << actions[0];
 
  for (int i = 1; i < actions.size(); ++i) {
    cout << ", " << actions[i];
  }
 
  cout << endl;
}

void
printActions(const vector<ConditionalActions>& actions) {
  cout << "Conditional actions are: ";
 
  int size = actions.size();
 
  if (size == 0) {
    cout << "NONE" << endl;
    return;
  }
 
  cout << actions[0].toLogString();
 
  for (int i = 1; i < actions.size(); ++i) {
    cerr << ", " << actions[i].toLogString();
  }
 
  cout << endl;
}


void printPurpose() {
  cerr << "Purpose: Use this program to check a user's access to a resource." << endl
       << endl;
}

void printUsage() {
  cerr << "Usage: CheckAccess args" << endl
       << "  -f arg             - akenti engine config file" << endl
       << "  -r arg             - resource " << endl
       << "  [-x arg]           - user's x509 cert chain" << endl
       << "  [-u arg -c arg]    - user's DN and user's CADN" << endl 
       << "  [-R arg]*          - a specific action(s) " << endl
       << "  [-A arg -V arg]*   - attribute/value pair" << endl
       << "  [-P]               - show all certs used in making the decision" << endl
       << "  [-o arg]           - certs' output file (defaults to stdout)" << endl
       << "  [-q]               - show concise output" << endl
       << "  [-v]               - verbose" << endl
       << "  [-h]               - prints this message" << endl;
}

int main(int argc, char** argv) {
  if (argc < 2) {
    printPurpose();
    printUsage();
    exit(2);
  }

  string confFileName;
  string x509CertFileName;
  string userDN;
  string caDN;
  string resource;
  string certOutputFile;
  set<string> desiredActions;
  multimap<string, string> desiredAttributes;
  string attribute;
  string value;
  bool detailed = false;
  bool verbose = false;
  bool concise = false;
  int opt;

  while ((opt = getopt(argc, argv, "r:x:u:c:f:o:A:V:R:Pvqh")) != EOF) {
    switch(opt) {
      case 'u':
        userDN = optarg;
        break;
      case 'c':
        caDN = optarg;
      case 'x':
        x509CertFileName = optarg;
        break;
      case 'f':
        confFileName = optarg;
        break;
      case 'r':
        resource = optarg;
        break;
      case 'o':
        certOutputFile = optarg;
        break;
      case 'R':
        desiredActions.insert(optarg);
        break;
      case 'A':
        attribute = optarg;
        break;
      case 'V':
        value = optarg;

        if (attribute.length() == 0) {
          cerr << "Must provide attribute and a value pair" << endl;
          exit(2);
        }

        desiredAttributes.insert(make_pair(attribute, value));
        attribute = value = "";
        break;
      case 'P':
        detailed = true;
        break;
      case 'q':
        concise = true;
        break;
      case 'v':
        verbose = true;
        break;
      case 'h':
      default:
        printUsage();
        exit(2);
    }
  }

  if (attribute.length() != 0) {
    cerr << "Must provide attribute and a value pair" << endl;
    exit(2);
  }

  try {
    if (!File(confFileName).isFile()) {
      cerr << "Can't locate engine config file " << confFileName << endl;
      exit(2);
    }

    if (verbose) { 
      cout << "Calling AkentInit::init with configuration file " << confFileName << endl;
    }

    string status;

    if (!AkentiInit::init(confFileName, status)) {
      cerr << "AkentiInit::init failed. " << status << endl;
      exit(2);
    }

    if (verbose) {
      cout << "Status from AkentiInit: "<< status << endl; 
    }

    if ((!userDN.length() || !caDN.length()) && (!x509CertFileName.length())) {
	cerr << "Must provide either (a userDN and a userCaDN) or a userX509CertFileleName" << endl;
	exit(2);
    } else if (!resource.length()) {
	cerr << "Must provide a resource" << endl;
	exit(2);
    }

    AkentiMessage msg;
    AkentiEngine eng;

    bool ret;

    eng.setDesiredActions(desiredActions);
    eng.setRestrictedAttributes(desiredAttributes);

    if (!detailed) {
      CapabilityCertificate capabilityCert;

      if (x509CertFileName != "") {
        if (verbose) {
          cout << "Reading user's cert chain" << endl; 
        }

        CertificateFactory certFac = CertificateFactory::getInstance();
        CertificateChain chain = certFac->initCertificateChainFromFile(
          "PEM", x509CertFileName
        );

        if (verbose) {
          cout << "Successfully read user's cert chain" << endl; 
          cout << "Calling eng.process for resource " << resource << endl;
        }

        ret = eng.checkAccess(resource, chain, capabilityCert, msg);
      } else {
        AkentiPrincipal user(userDN, caDN);

        if (verbose) {
          cout << "User DN is " << userDN << " CaDN is "<< caDN << endl;
          cout << "Calling eng.process for resource " << resource << endl;
        }

        ret = eng.checkAccess(resource, user, capabilityCert, msg);
      }
 
      cout << msg.toString() << endl;

      if (ret && concise) {
        printActions(capabilityCert->getUnConditionalActions());
        printActions(capabilityCert->getConditionalActions());
      } else if (ret) {
	if (certOutputFile.length() == 0) {
	  AkentiCertificate::writeToStream("XML", cout, capabilityCert);
        } else {
	  AkentiCertificate::writeToFile("XML", certOutputFile, capabilityCert);
        }
      }
    } else { 
      PolicyContext ctx;

      if (x509CertFileName != "") {
        if (verbose) {
          cout << "Reading user's cert chain:" << x509CertFileName << endl; 
        }

        CertificateFactory certFac = CertificateFactory::getInstance();
        CertificateChain chain = certFac->initCertificateChainFromFile(
          "PEM", x509CertFileName
        );

        if (verbose) {
          cout << "Successfully read user's cert chain" << endl; 
          cout << "Calling engine for resource " << resource << endl;
        }

        ret = eng.checkAccess(resource, chain, ctx, msg);
      } else {
        AkentiPrincipal user(userDN, caDN);

        if (verbose) {
          cout << "User DN is " << userDN << ", CaDN is "<< caDN << endl;
          cout << "Calling engine for resource " << resource << endl;
        }

        ret = eng.checkAccess(resource, user, ctx, msg);
      }

      bool haveOutputFile = certOutputFile.length() != 0;
      ostream* os = haveOutputFile? new ofstream(certOutputFile.c_str()) : &cout;
      const vector<Policy>& policies = ctx->getPolicies();
        
      for (int i = 0; i < policies.size(); ++i) {
        const PolicyCertificate& cert = policies[i]->getPolicyCertificate();

	AkentiCertificate::writeToStream("XML", *os, cert);

        const vector<UseCondCertificate>& ucCerts = policies[i]->getUCCerts();

        for (int j = 0; j < ucCerts.size(); ++j) {
	  AkentiCertificate::writeToStream("XML", *os, ucCerts[j]);
        }
      }
        
      const vector<AttributeCertificate>& attrCerts = ctx->getAttributeCerts();
        
      for (int k = 0; k < attrCerts.size(); ++k) {
        AkentiCertificate::writeToStream("XML", *os, attrCerts[k]);
      }
        
      if (ret) {
        const CapabilityCertificate& capabilityCert = ctx->getCapabilityCert();

        AkentiCertificate::writeToStream("XML", *os, capabilityCert);
      }  

      if (haveOutputFile) {
        delete(os);
      }

      cout << msg.toString() << endl;
    }

    exit(!ret);
  } catch(Exception& ex) {
    cerr << "EXCEPTION:" << ex << endl;
    exit(3);
  } catch (Error& err) {
    cerr << "ERROR:Report to Akenti team:" <<  err << endl;
    exit(4);
  }
}

}
