#ifndef AKENTI_PRINCIPAL_HPP
#define AKENTI_PRINCIPAL_HPP

#include "AkentiObject.hpp"
#include "DistinguishedName.hpp"

namespace Akenti {

/**
  * An <i>AkentiPrincipal</i> is used to represent an entity
  * in the <i>Akenti</i> world. It consists of two components
  * the name of the entity and the name of the guarantor of that
  * entity.
  *                                                   <BR>
  * Note:                                             <BR>
  * 
  * The names are <i>x509</i> dns.                    <BR>
  * The guarantor is the immediate ca of the entity.  <BR>

  * <pre>
  * 
  * AkentiPrincipal {             
  *   DistinguishedName name     
  *   DistinguishedNmae guarantor  
  * }                             
  *
  * Format:
  *   name guarantor     
  *
  * Example: (/C=US/ST=CA/CN=John Doe, /C=US/ST=CA/CN=CA)
  *    
  *   will be writen as: /C=US/ST=CA/CN=John\ Doe /C=US/ST=CA/CN=CA
  *
  * </pre>
  *
  * @see AkentiObject
  * @see DistinguishedName 
  *
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  **/

class AkentiPrincipal : public AkentiObject {

public:

  /**
    * Default Constructor.
    **/
  AkentiPrincipal();
 
  /**
    * Constructs this object.
    *
    * @see DistinguishedName 
    **/
  AkentiPrincipal(const DistinguishedName& name, 
                  const DistinguishedName& guarantor);

  /**
    * Constructs this object.
    *
    * @see DistinguishedName 
    **/
  AkentiPrincipal(const string& name, const string& guarantor);

  /** 
    * Destroys this object.
    **/
  ~AkentiPrincipal();

  /**
    * Assignement operator.
    */
  void operator=(const AkentiPrincipal& ap);

  /**
    * Returns <i>true</i> if equal.
    **/
  bool operator==(const AkentiPrincipal& ap) const;

  /**
    * Returns name of the entity.
    **/
  const DistinguishedName& getName() const;

  /**
    * Returns the name of the guarantor of the entity. 
    **/
  const DistinguishedName& getGuarantor() const;
 
  /**
    * Writes this object to an <i>AkentiOutputStream</i>.
    *
    * @see AkentiOutputStream
    **/
  void writeObject(AkentiOutputStream& drain) const;

  /**
    * Reads this object from the <i>AkentiInputStream</i> object.
    *
    * @see AkentiInputStream
    **/
  void readObject(AkentiInputStream& source);

  /**
    * Populates the XML element <i>ap</i> from this <i>AkentiPrincipal</i>.
    */
  void toXML(DOMElement* ap) const;

  /**
    * Initializes this <i>AkentiPrincipal</i> from the XML element
    * <i>ap</i>.
    */
  void fromXML(const DOMElement* ap);

  /**
    * Used for debugging.
    */
  string paramString() const;

  /**
    * Used for logging. 
    */
  string toLogString() const;

private:

  DistinguishedName name;         // distinguished name of the entity

  DistinguishedName guarantor;    // distinguished name of the guarantor of the entity

};

} // end of name space

#endif
